@@ -18,7 +18,6 @@
 #include <math.h>
 #include <qpainter.h>
 
-#include "mt/MutexGuard.h"
 #include "libkwave/SampleReader.h"
 #include "libkwave/Track.h"
 #include "libgui/TrackPixmap.h"
@@ -73,7 +72,7 @@ TrackPixmap::TrackPixmap(Track &track)
 //***************************************************************************
 TrackPixmap::~TrackPixmap()
 {
-    MutexGuard lock(m_lock_buffer);
+    QMutexLocker lock(&m_lock_buffer);
 
     if (m_interpolation_alpha) delete[] m_interpolation_alpha;
     m_interpolation_alpha = 0;
@@ -82,7 +81,7 @@ TrackPixmap::~TrackPixmap()
 //***************************************************************************
 void TrackPixmap::setOffset(unsigned int offset)
 {
-    MutexGuard lock(m_lock_buffer);
+    QMutexLocker lock(&m_lock_buffer);
     if (offset == m_offset) return; // no change
 
     unsigned int diff;
@@ -203,7 +202,7 @@ void TrackPixmap::resizeBuffer()
 //***************************************************************************
 void TrackPixmap::setZoom(double zoom)
 {
-    MutexGuard lock(m_lock_buffer);
+    QMutexLocker lock(&m_lock_buffer);
 
     if (zoom == m_zoom) return; // no change
 
@@ -236,7 +235,7 @@ void TrackPixmap::setZoom(double zoom)
 //***************************************************************************
 void TrackPixmap::resize(int width, int height)
 {
-    MutexGuard lock(m_lock_buffer);
+    QMutexLocker lock(&m_lock_buffer);
 
     int old_width = QPixmap::width();
     int old_height = QPixmap::height();
@@ -379,7 +378,7 @@ bool TrackPixmap::validateBuffer()
 //***************************************************************************
 void TrackPixmap::repaint()
 {
-    MutexGuard lock(m_lock_buffer);
+    QMutexLocker lock(&m_lock_buffer);
 
     int w = width();
     int h = height();
@@ -425,7 +424,7 @@ void TrackPixmap::repaint()
 //***************************************************************************
 bool TrackPixmap::isModified()
 {
-    MutexGuard lock(m_lock_buffer);
+    QMutexLocker lock(&m_lock_buffer);
     return m_modified;
 }
 
@@ -442,11 +441,22 @@ void TrackPixmap::drawOverview(QPainter &p, int middle, int height,
     int max = 0, min = 0;
 
     p.setPen(m_color_sample);
+    int last_min = (int)(m_min_buffer[first] * scale_y);
+    int last_max = (int)(m_max_buffer[first] * scale_y);
     for (int i = first; i <= last; i++) {
 	Q_ASSERT(m_valid[i]);
 	max = (int)(m_max_buffer[i] * scale_y);
 	min = (int)(m_min_buffer[i] * scale_y);
+
+	// make sure there is a connection between this
+	// section and the one before, avoid gaps
+	if (min > last_max+1) min = last_max+1;
+	if (max+1 < last_min) max = last_min-1;
+
 	p.drawLine(i, middle - max, i, middle - min);
+
+	last_min = min;
+	last_max = max;
     }
 }
 
@@ -668,13 +678,19 @@ void TrackPixmap::drawPolyLineSignal(QPainter &p, int width,
 	points->setPoint(i++, x, middle - y);
     }
 
-    // show the poly-line
-    p.setPen(darkGray);
-    p.drawPolyline(*points, 0, i);
+    if (m_zoom >= 1.0) {
+	// show only poly-line (bright)
+	p.setPen(white);
+	p.drawPolyline(*points, 0, i);
+    } else {
+	// show the poly-line (dark)
+	p.setPen(darkGray);
+	p.drawPolyline(*points, 0, i);
 
-    // show the original points
-    p.setPen(white);
-    p.drawPoints(*points, 0, n);
+	// show the original points (bright)
+	p.setPen(white);
+	p.drawPoints(*points, 0, n);
+    }
 
     delete points;
 }
@@ -684,7 +700,7 @@ void TrackPixmap::slotSamplesInserted(Track &, unsigned int offset,
                                       unsigned int length)
 {
     {
-	MutexGuard lock(m_lock_buffer);
+	QMutexLocker lock(&m_lock_buffer);
 
 	convertOverlap(offset, length);
 	if (!length) return; // false alarm
@@ -708,7 +724,7 @@ void TrackPixmap::slotSamplesDeleted(Track &, unsigned int offset,
                                      unsigned int length)
 {
     {
-	MutexGuard lock(m_lock_buffer);
+	QMutexLocker lock(&m_lock_buffer);
 
 	convertOverlap(offset, length);
 	if (!length) return; // false alarm
@@ -732,7 +748,7 @@ void TrackPixmap::slotSamplesModified(Track &, unsigned int offset,
                                       unsigned int length)
 {
     {
-	MutexGuard lock(m_lock_buffer);
+	QMutexLocker lock(&m_lock_buffer);
 
 	convertOverlap(offset, length);
 	if (!length) return; // false alarm
