@@ -13,7 +13,7 @@
  * \date    2000-09-30 (ported to the Kwave project)
  *
  * \par NOTE:
- *      this code is taken from the following news article 
+ *      this code is taken from the following news article
  *      (found by www.dejanews.com):
  *
  * \verbatim
@@ -41,13 +41,15 @@
 #include <error.h>   // for strerror()
 #include <time.h>    // for clock()
 #include <qglobal.h> // for qWarning()
+#include <qmutex.h>
 #include <signal.h>
+#include <sys/time.h>
 
-#include "mt/TSS_Object.h"
-#include "mt/MutexGuard.h"
-#include "mt/Mutex.h"
 #include "mt/Thread.h"
 
+/** lock for protecting SIGHUP <-> thread exit */
+static QMutex g_lock_sighup;
+
 //***************************************************************************
 extern "C" void _dummy_SIGHUP_handler(int)
 {
@@ -59,7 +61,7 @@ extern "C" void _dummy_SIGHUP_handler(int)
  * C wrapper function for Thread::thread_adapter.
  * @internal
  */
-extern "C" void* C_thread_adapter(void* arg)
+extern "C" void *C_thread_adapter(void* arg)
 {
     Thread *thread = (Thread *)arg;
     Q_CHECK_PTR(thread);
@@ -69,20 +71,21 @@ extern "C" void* C_thread_adapter(void* arg)
     sighandler_t old_handler = signal(SIGHUP, _dummy_SIGHUP_handler);
 
     /* call the thread's function through a C++ adapter */
-    void* result = thread->thread_adapter(arg);
+    void *result = thread->thread_adapter(arg);
 
     /* restore previous signal handler */
     old_handler = signal(SIGHUP, old_handler);
 
+    g_lock_sighup.unlock();
     return result;
 }
 
 //***************************************************************************
 Thread::Thread(int */*grpid*/, const long /*flags*/)
-    :QObject(), TSS_Object(), m_tid((pthread_t)-1), m_lock("thread"),
-    m_thread_running("thread_running"), m_should_stop(false)
+    :QObject(), m_tid((pthread_t)-1), m_lock(),
+    m_thread_running(), m_should_stop(false)
 {
-    MutexGuard lock(m_lock);
+    QMutexLocker lock(&m_lock);
     int res;
     res = pthread_attr_init(&m_attr);
     if (res)
@@ -115,21 +118,26 @@ Thread::~Thread()
 //***************************************************************************
 void *Thread::thread_adapter(void *arg)
 {
-    MutexGuard lock(m_thread_running);
+    QMutexLocker lock(&m_thread_running);
 
     Thread *object = (Thread *) arg;
     Q_CHECK_PTR(object);
-    if (!object) return (void*)-EINVAL;
+    if (!object) {
+	g_lock_sighup.lock();
+	return (void*)-EINVAL;
+    }
 
     /* execute the thread function */
     object->run();
+
+    g_lock_sighup.lock();
     return arg;
 }
 
 //***************************************************************************
 int Thread::start()
 {
-    MutexGuard lock(m_lock);
+    QMutexLocker lock(&m_lock);
 
     // reset the "should stop" command flag
     m_should_stop = false;
@@ -144,20 +152,35 @@ int Thread::start()
 //***************************************************************************
 int Thread::stop(unsigned int timeout)
 {
-    MutexGuard lock(m_lock);
+    QMutexLocker lock(&m_lock);
     if (!running()) return 0; // already down
 
     if (timeout < 1000) timeout = 1000;
 
-    // try to stop cooperatively
+    // set the "should stop" flag
     m_should_stop = true;
+
+    // send one SIGHUP in advance
+    {
+	QMutexLocker lock_exit(&g_lock_sighup);
+	if (!running()) return 0;
+	pthread_kill(m_tid, SIGHUP);
+    }
+
+    // try to stop cooperatively
+    if (!running()) return 0;
     wait(timeout/10);
     if (!running()) return 0;
 
     // try to interrupt by INT signal
     qDebug("Thread::stop(): sending SIGHUP");
     for (unsigned int i=0; i < 8; i++) {
-	pthread_kill(m_tid, SIGHUP);
+	{
+	    QMutexLocker lock_exit(&g_lock_sighup);
+	    if (!running()) return 0;
+	    pthread_kill(m_tid, SIGHUP);
+	}
+	if (!running()) return 0;
 	wait(timeout/10);
 	if (!running()) return 0;
     }
@@ -189,15 +212,25 @@ bool Thread::running()
 void Thread::wait(unsigned int milliseconds)
 {
     double elapsed_ms = 0.0;
-    time_t t_start = clock();
+    struct timeval t1, t2;
+    gettimeofday(&t1, 0);
 
     while (running() && (elapsed_ms < milliseconds)) {
+	// just for fun...
 	sched_yield();
-	elapsed_ms = (((double)(clock()-t_start))/CLOCKS_PER_SEC)*1000.0;
+
+	// sleep through select()
+	struct timeval tv;
+	tv.tv_sec = 0;
+	tv.tv_usec = 20*1000;
+	select(0, 0, 0, 0, &tv);
+
+	gettimeofday(&t2, 0);
+	elapsed_ms = (double)t2.tv_sec  * 1.0E3 +
+	             (double)t2.tv_usec / 1.0E3 -
+	             (double)t1.tv_sec  * 1.0E3 -
+	             (double)t1.tv_usec / 1.0E3;
     }
-//    if (/*still */running()) {
-//	qWarning("Thread::wait(): timed out after %d ms!", milliseconds);
-//    }
 }
 
 //***************************************************************************
