@@ -15,11 +15,15 @@
  *                                                                         *
  ***************************************************************************/
 
+#include "config.h"
+#ifdef HAVE_ARTS_SUPPORT
+
 #include <errno.h>
 #include <math.h>
-#include "mt/MutexGuard.h"
-#include "mt/ThreadsafeX11Guard.h"
+#include <pthread.h>
 #include <klocale.h>
+
+#include "mt/ThreadsafeX11Guard.h"
 #include "PlayBack-aRts.h"
 
 /** use at least 2^8 = 256 bytes for playback buffer */
@@ -32,7 +36,7 @@
 static int g_arts_usage = 0;
 
 //***************************************************************************
-PlayBackArts::PlayBackArts(Mutex &arts_lock)
+PlayBackArts::PlayBackArts(QMutex &arts_lock)
     :PlayBackDevice(),
     m_stream(0),
     m_rate(0),
@@ -67,7 +71,7 @@ QString PlayBackArts::open(const QString &, double rate,
     m_buffer_size = 0;
     m_buffer_used = 0;
     m_stream      = 0;
-    MutexGuard lock(m_lock_aRts);
+    QMutexLocker lock(&m_lock_aRts);
 
     Q_ASSERT(m_closed);
     m_closed = false;
@@ -115,12 +119,12 @@ int PlayBackArts::write(QMemArray<sample_t> &samples)
 	return -EIO;
     }
     Q_ASSERT(samples.count() == m_channels);
-    
+
     // convert into byte stream
     unsigned int channel;
     for (channel=0; channel < m_channels; channel++) {
 	sample_t sample = samples[channel];
-	
+
 	switch (m_bits) {
 	    case 8:
 		sample += 1 << 23;
@@ -157,11 +161,11 @@ void PlayBackArts::flush()
 
     Q_ASSERT(m_stream);
     if (m_stream) {
-	MutexGuard lock(m_lock_aRts);
+	QMutexLocker lock(&m_lock_aRts);
 
-	// fill rest of buffer with zeroes	
+	// fill rest of buffer with zeroes
 	while (m_buffer_used < m_buffer_size)
-	
+
 	    m_buffer[m_buffer_used++] = 0;
 	int errorcode = arts_write(m_stream, m_buffer.data(), m_buffer_used);
 	if (errorcode < 0) {
@@ -180,17 +184,17 @@ int PlayBackArts::close()
     flush();
 
     {
-	MutexGuard lock(m_lock_aRts);
-	
+	QMutexLocker lock(&m_lock_aRts);
+
 	// close the playback stream
 	if (m_stream) arts_close_stream(m_stream);
-	
+
 	if (!m_closed && g_arts_usage && !--g_arts_usage) {
 	    qDebug("PlayBackArts::close() [pid %d]: releasing aRts API",
 	          (int)pthread_self());
 	    arts_free();
 	}
-	
+
 	m_stream = 0;
 	m_closed = true;
     }
@@ -233,5 +237,40 @@ QString PlayBackArts::artsErrorText(int errorcode)
     return text;
 };
 
+//***************************************************************************
+QStringList PlayBackArts::supportedDevices()
+{
+    QStringList list; // list stays empty, no selectable devices
+    list.append(i18n("[aRts sound daemon]"));
+    return list;
+}
+
+//***************************************************************************
+QValueList<unsigned int> PlayBackArts::supportedBits(const QString &device)
+{
+    (void)device;
+    QValueList <unsigned int> bits;
+
+    // aRts supports only 8 and 16 bits as far as I know...
+    bits.append(8);
+    bits.append(16);
+
+    return bits;
+}
+
+//***************************************************************************
+int PlayBackArts::detectChannels(const QString &device,
+                                 unsigned int &min, unsigned int &max)
+{
+    (void) device;
+
+    // aRts supports only mono and stereo as far as I know...
+    min = 1;
+    max = 2;
+    return 0;
+}
+
+#endif /* HAVE_ARTS_SUPPORT */
+
 //***************************************************************************
 //***************************************************************************
